@@ -1,5 +1,5 @@
 package ETL::Yertl::Adapter::graphite;
-our $VERSION = '0.035';
+our $VERSION = '0.036';
 # ABSTRACT: Adapter to read/write from Graphite time series database
 
 #pod =head1 SYNOPSIS
@@ -40,6 +40,7 @@ use JSON::MaybeXS qw( decode_json );
 use List::Util qw( first );
 use IO::Async::Loop;
 use Time::Piece ();
+use Scalar::Util qw( looks_like_number );
 
 #pod =method new
 #pod
@@ -192,8 +193,8 @@ sub read_ts {
 #pod
 #pod =item timestamp
 #pod
-#pod An ISO8601 timestamp. Optional. Defaults to the current time on the
-#pod InfluxDB server.
+#pod An ISO8601 timestamp or UNIX epoch time. Optional. Defaults to the
+#pod current time.
 #pod
 #pod =item value
 #pod
@@ -213,10 +214,13 @@ sub write_ts {
     my $sock = $self->write_client;
     for my $point ( @points ) {
         die "Tags are not supported by Graphite" if $point->{tags} && keys %{ $point->{tags} };
+        $point->{timestamp} ||= time;
         $point->{timestamp} =~ s/[.]\d+Z?$//; # We do not support nanoseconds
+        if ( !looks_like_number( $point->{timestamp} ) ) {
+            $point->{timestamp} = Time::Piece->strptime( $point->{timestamp}, '%Y-%m-%dT%H:%M:%S' )->epoch;
+        }
         $sock->write(
-            join( " ", $point->{metric}, $point->{value},
-            Time::Piece->strptime( $point->{timestamp}, '%Y-%m-%dT%H:%M:%S' )->epoch, )
+            join( " ", $point->{metric}, $point->{value}, $point->{timestamp}, )
             . "\n",
         );
     }
@@ -231,6 +235,10 @@ sub write_ts {
 # HH:MM_YYYYMMDD
 sub _format_graphite_dt {
     my ( $iso ) = @_;
+    if ( looks_like_number( $iso ) ) {
+        my $t = Time::Piece->new( $iso );
+        return sprintf "%s:%s_%s%s%s", $t->hour, $t->min, $t->year, $t->mon, $t->mday;
+    }
     if ( $iso =~ /^(\d{4})-?(\d{2})-?(\d{2})$/ ) {
         return join "", $1, $2, $3;
     }
@@ -250,7 +258,7 @@ ETL::Yertl::Adapter::graphite - Adapter to read/write from Graphite time series
 
 =head1 VERSION
 
-version 0.035
+version 0.036
 
 =head1 SYNOPSIS
 
@@ -333,8 +341,8 @@ The metric to write. For Graphite, this is a path separated by dots
 
 =item timestamp
 
-An ISO8601 timestamp. Optional. Defaults to the current time on the
-InfluxDB server.
+An ISO8601 timestamp or UNIX epoch time. Optional. Defaults to the
+current time.
 
 =item value
 
