@@ -1,10 +1,12 @@
 package ETL::Yertl::Command::yq::Regex;
-our $VERSION = '0.035';
+our $VERSION = '0.036';
 # ABSTRACT: A regex-based parser for programs
 
 use ETL::Yertl;
 use boolean qw( :all );
 use Regexp::Common;
+use Time::Local qw( timegm );
+use ETL::Yertl::Util qw( firstidx );
 
 sub empty() {
     bless {}, 'empty';
@@ -34,15 +36,15 @@ our $GRAMMAR = qr{
             \w+ # Constant/bareword
         )
         (?<OP>eq|ne|==?|!=|>=?|<=?)
-        (?<FUNC_NAME>empty|select|grep|group_by|keys|length|sort|each)
+        (?<FUNC_NAME>empty|select|grep|group_by|keys|length|sort|each|parse_time)
         (?<EXPR>
-            \{(\s*(?&FILTER)\s*:\s*(?0)\s*(?:,(?-1))*)\} # Hash constructor
+            \{\s*(?&FILTER)\s*:\s*(?0)\s*(?:,(?-1))*\} # Hash constructor
             |
-            \[(\s*(?0)\s*(?:,(?-1))*)\] # Array constructor
+            \[\s*(?0)\s*(?:,(?-1))*\] # Array constructor
             |
-            (?&FUNC_NAME)(?:\(\s*((?&EXPR))\s*\))? # Function with optional argument
+            (?&FUNC_NAME)(?:\(\s*(?&EXPR)\s*(?:,\s*(?&EXPR)\s*)*\))? # Function with optional argument(s)
             |
-            (?:(?&FILTER)|(?&FUNC_NAME)(?:\(\s*((?&EXPR))\s*\))?)\s+(?&OP)\s+(?&EXPR) # Binop with filter
+            (?:(?&FILTER)|(?&FUNC_NAME)(?:\(\s*(?&EXPR)\s*\))?)\s+(?&OP)\s+(?&EXPR) # Binop with filter
             |
             (?&FILTER)
         )
@@ -55,6 +57,17 @@ my $FUNC_NAME = qr{(?&FUNC_NAME)$GRAMMAR};
 my $EXPR = qr{(?&EXPR)$GRAMMAR};
 my $PIPE = qr{[|]};
 
+my @DAYS = qw< sun sunday mon monday tue tuesday wed wednesday thu thursday fri friday sat saturday sun sunday>;
+my $DAYS = qr{@{[ join '|', @DAYS ]}}i;
+my @MONTHS = qw< jan feb mar apr may jun jul aug sep oct nov dec >;
+my $MONTHS = qr{@{[ join '|', @MONTHS ]}}i;
+
+my %PARSE_TIME = (
+    iso => qr{(?<y>\d{4})-?(?<m>\d{2})-?(?<d>\d{2})(?:[ T]?(?<h>\d{2}):?(?<n>\d{2})(?::?(?<s>\d{2})))?},
+    apache => qr{(?<d>\d{2})/(?<mn>$MONTHS)/(?<y>\d{4}):(?<h>\d{2}):(?<n>\d{2}):(?<s>\d{2})},
+);
+$PARSE_TIME{auto} = qr{$PARSE_TIME{iso}|$PARSE_TIME{apache}};
+
 # Filter MUST NOT mutate $doc!
 sub filter {
     my ( $class, $filter, $doc, $scope, $orig_doc ) = @_;
@@ -73,6 +86,7 @@ sub filter {
         }
         return @in;
     }
+
     # Hash constructor
     elsif ( $filter =~ /^{/ ) {
         my %out;
@@ -84,6 +98,7 @@ sub filter {
         }
         return \%out;
     }
+
     # Array constructor
     elsif ( $filter =~ /^\[/ ) {
         my @out;
@@ -93,42 +108,37 @@ sub filter {
         }
         return \@out;
     }
-    # , does multiple filters, yielding multiple documents
-    elsif ( $filter =~ /,/ ) {
-        my @filters = split /\s*,\s*/, $filter;
-        return map { $class->filter( $_, $doc, $scope, $orig_doc ) } @filters;
-    }
+
     # Function calls
-    elsif ( $filter =~ /^((?&FUNC_NAME))(?:\(\s*((?&EXPR))\s*\))?$GRAMMAR$/ ) {
-        my ( $func, $expr ) = ( $1, $2 );
-        diag( 1, "F: $func, ARG: " . ( $expr || '' ) );
+    elsif ( my ( $func, @args ) = $filter =~ /^((?&FUNC_NAME))(?:\(\s*((?&EXPR))\s*(?:,\s*((?&EXPR))\s*)*\))?$GRAMMAR$/ ) {
+        diag( 1, "F: $func, ARGS: " . ( join( ', ', grep defined, @args ) || '' ) );
         if ( $func eq 'empty' ) {
-            if ( $expr ) {
+            if ( @args ) {
                 warn "empty does not take arguments\n";
             }
             return empty;
         }
         elsif ( $func eq 'select' || $func eq 'grep' ) {
-            if ( !$expr ) {
+            if ( !@args ) {
                 warn "'$func' takes an expression argument";
                 return empty;
             }
-            return $class->filter( $expr, $doc, $scope, $orig_doc ) ? $doc : empty;
+            return $class->filter( $args[0], $doc, $scope, $orig_doc ) ? $doc : empty;
         }
         elsif ( $func eq 'group_by' ) {
-            my $grouping = $class->filter( $expr, $doc, $scope, $orig_doc );
+            my $grouping = $class->filter( $args[0], $doc, $scope, $orig_doc );
             push @{ $scope->{ group_by }{ $grouping } }, $doc;
             return;
         }
         elsif ( $func eq 'sort' ) {
-            $expr ||= '.';
-            my $value = $class->filter( $expr, $doc, $scope, $orig_doc );
+            $args[0] ||= '.';
+            my $value = $class->filter( $args[0], $doc, $scope, $orig_doc );
             push @{ $scope->{sort} }, [ "$value", $doc ];
             return;
         }
         elsif ( $func eq 'keys' ) {
-            $expr ||= '.';
-            my $value = $class->filter( $expr, $doc, $scope, $orig_doc );
+            $args[0] ||= '.';
+            my $value = $class->filter( $args[0], $doc, $scope, $orig_doc );
             if ( ref $value eq 'HASH' ) {
                 return [ keys %$value ];
             }
@@ -141,8 +151,8 @@ sub filter {
             }
         }
         elsif ( $func eq 'each' ) {
-            $expr ||= '.';
-            my $value = $class->filter( $expr, $doc, $scope, $orig_doc );
+            $args[0] ||= '.';
+            my $value = $class->filter( $args[0], $doc, $scope, $orig_doc );
             if ( ref $value eq 'HASH' ) {
                 return map +{ key => $_, value => $value->{ $_ } }, keys %$value;
             }
@@ -155,8 +165,8 @@ sub filter {
             }
         }
         elsif ( $func eq 'length' ) {
-            $expr ||= '.';
-            my $value = $class->filter( $expr, $doc, $scope, $orig_doc );
+            $args[0] ||= '.';
+            my $value = $class->filter( $args[0], $doc, $scope, $orig_doc );
             if ( ref $value eq 'HASH' ) {
                 return scalar keys %$value;
             }
@@ -171,7 +181,28 @@ sub filter {
                 return empty;
             }
         }
+        elsif ( $func eq 'parse_time' ) {
+            my ( $expr, $format ) = @args;
+            $format ||= 'auto';
+            die sprintf "Invalid format '%s' in parse_time()\n", $format
+                if !$PARSE_TIME{ $format};
+            my $value = $class->filter( $expr, $doc, $scope, $orig_doc );
+            diag( 1, "FMT: $PARSE_TIME{ $format }, VAL: $value" );
+            if ( $value =~ $PARSE_TIME{ $format } ) {
+                my @tlargs = @{+}{qw< s n h d m y >};
+                if ( !$+{m} && ( my $mname = $+{mn} ) ) {
+                    $tlargs[4] = firstidx { /$mname/i } @MONTHS;
+                }
+                else {
+                    $tlargs[4] -= 1;
+                }
+                return timegm( @tlargs );
+            }
+            warn sprintf "time '%s' does not match format '%s'\n", $value, $format;
+            return empty;
+        }
     }
+
     # Hash and array keys to traverse the data structure
     elsif ( $filter =~ /^((?&FILTER))$GRAMMAR$/ ) {
         # Extract quoted strings
@@ -275,6 +306,7 @@ sub filter {
             }
         }
     }
+
     # Conditional (if/then/else)
     # NOTE: If we're capturing using $EXPR, then we _must_ use named captures,
     # because $EXPR has captures in itself
@@ -288,6 +320,17 @@ sub filter {
             return $false_filter ? $class->filter( $false_filter, $doc, $scope, $orig_doc ) : ();
         }
     }
+
+    # , does multiple filters, yielding multiple documents
+    # This must be the least-specific rule because of all the other
+    # possible uses of the comma
+    # XXX: In the future, this should be used to parse function
+    # arguments to allow for recursion
+    elsif ( $filter =~ /,/ ) {
+        my @filters = split /\s*,\s*/, $filter;
+        return map { $class->filter( $_, $doc, $scope, $orig_doc ) } @filters;
+    }
+
     else {
         die "Could not parse filter '$filter'\n";
     }
@@ -306,7 +349,7 @@ ETL::Yertl::Command::yq::Regex - A regex-based parser for programs
 
 =head1 VERSION
 
-version 0.035
+version 0.036
 
 =head1 AUTHOR
 
